@@ -7,7 +7,7 @@ use strict;
 
 package XML::Compile::Transport::SOAPHTTP;
 use vars '$VERSION';
-$VERSION = '3.12';
+$VERSION = '3.13';
 
 use base 'XML::Compile::Transport';
 
@@ -257,7 +257,7 @@ multipart/related;
 __CT
 
         my $base = HTTP::Message->new
-          ( [ Content_Type => qq{$mime_xop; charset="$charset"; type="$si"}
+          ( [ Content_Type => qq{$mime_xop; charset="$charset"; type="$content_type"}
             , Content_Transfer_Encoding => '8bit'
             , Content_ID  => "<$start_cid>"
             ] );
